/*
 * Copyright (c) 2001 Falk Feddersen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <glib.h>

#include "defs.h"
#include "cmdline.h"



void help_report()
{

  fprintf(stderr,"kepsilon:  Surfzone Turbulence and Bubble Model  Version %1d.%d.%d\n",
                  MAJOR_VERSION_NUMBER,MINOR_VERSION_NUMBER,MICRO_VERSION_NUMBER);
  fprintf(stderr,"(c) 2002 Falk Feddersen\n");
  fprintf(stderr,"Usage: kepsilon [OPTIONS] [init_file]\n");
  fprintf(stderr,"Options:\n");
  fprintf(stderr,"\t -v : verbose output\n");
  fprintf(stderr,"\t -p : parse init file\n");
  fprintf(stderr,"\t -h : this help\n");
  fprintf(stderr,"\t -d <num>: diagnostic output, memory & numerical stability. <num>=1,2,3.\n");
  fprintf(stderr,"\t -g : GUI time output\n");
  fprintf(stderr,"\t -m <msg>: GUI message displayed in GUI window.\n");

  fprintf(stderr,"\n If no init_file is specified, 'default.init' is used as default.\n");
  fprintf(stderr,"Init file format:\n\n");
  fprintf(stderr,"  nz dz  z0b z0s    %% this is a comment\n");
  fprintf(stderr,"  [craig,kepsilon,q2l] - which type of TKE model\n");
  fprintf(stderr,"  tke  [0,1] tke_ic  tke_bc?? -> this needs updating\n");
  fprintf(stderr,"  bubble [on,off] r_min r_max nr (ic beta0 H_scale)\n");
  fprintf(stderr,"  wave [on,off](waves)  [on,off](breaking) Hrms T theta [epsilon_wave]\n");
  fprintf(stderr,"  mflow [on,off] U_*wind  V_*wind  [surfzone,(Fx Fy)]\n");
  fprintf(stderr,"  density [on,off]  [flux,fixed] bcval N\n");
  fprintf(stderr,"  sediment [on,off]  D50\n");
  fprintf(stderr,"  timing  time3 units  time2 units time1 units dt units - units=[hr,min,sec]\n");
  fprintf(stderr,"  Output stuff: see manual\n");

  exit(-1);

}


cmdline* process_args2(int argc,char *argv[])
{

  int c,dd,index;
  cmdline *A;
  char *dval=NULL;


  /* set getopt error reporting to nill */

  opterr = 0;


  /* initialize the cmdline A */

  A = (cmdline * ) malloc(sizeof(cmdline));
  //  strcpy(A->init_file,"init.dat");
  A->init_file = g_string_new("default.init");
  A->message = NULL;
  A->verbose=0;
  A->diagnostic=0;
  A->help = 0;
  A->parse = 0;
  A->gui = 0;
  A->msg = 0;

  while ((c = getopt (argc, argv, "vhpgm:d:")) != -1)
    switch (c)  {
      case 'v':  A->verbose=1;  break;
      case 'd': dval = optarg;
        A->diagnostic=3;
        if (dval) {
          if (!isdigit(dval[0])) {
	    fprintf (stderr,"** kepsilon: Bad argument to diagnostics flag -d\n");
	    exit(-1);
	  }
          dd = atoi(dval);
          switch (dd) {
  	  case 1: A->diagnostic=1; break;
	  case 2: A->diagnostic=2; break;
	  case 3: A->diagnostic=3; break;
	  default: A->diagnostic=1;  //fprintf(stderr,"Bad option to diagnostics flag -d\n"); exit(-1);
	  }
	}
        break;
      case 'h':  A->help=1; break;
      case 'p':  A->parse=1; break;
      case 'g':  A->gui=1; break;
      case 'm': 
        if (optarg[0]=='-') {
          fprintf (stderr,"** kepsilon: Bad argument to message option\n");
          exit(-1);
	}
        A->msg=1;
        A->message = g_string_new(optarg);
        break;

      case '?':
        if (optopt=='d') {
	  A->diagnostic=3;
          break;
	}
        if (isprint (optopt))
          fprintf (stderr, "** kepsilon: Unknown option `-%c'.\n", optopt);
        else
          fprintf (stderr,"** kepsilon: Unknown option character `\\x%x'.\n",optopt);
      default:
        exit (-1);

      }

  index = optind;
  if (index<argc) {
      A->init_file = g_string_assign(A->init_file,argv[index]);
  //      strcpy(A->init_file,argv[index]);
  }

  if (A->help==1) 
    help_report();

  return A;

}
