/*  Falk Feddersen
    field.h

    C code to make an array (field) data structure

*/


#ifndef FIELD_H
#define FIELD_H

#ifdef __cplusplus
extern "C" {
#endif				/* __cplusplus */


/* header file field.h  */


typedef struct  {
  int N;
  double *data;
} field1D;


typedef struct  {
  int N,M,num;
  double *data;
  double **row;
} field2D;

/* defines for dereferencing */

#define DR1(T,i)  ((T)->data[(i)])

#define DR2(T,i,j)  ((T)->data[(i)*((T)->M)+(j)])
#define DR2a(T,k)   ((T)->data[(k)])       // this is for when k=i*M+j

#define REF1(T)  ((T)->data)
#define REF2(T,i) ((T)->row[(i)])


/* field1D function declarations */

field1D*  allocate_field1D( int );
void deallocate_field1D(field1D * );
int  set_field1D_to_value(field1D *T, double a);
int  copy_field1D(field1D *T, const field1D *S);        // copy S into T
void mult_field1D_const(field1D *T, double a);
int copy_field1D_mult(field1D *T, const field1D *S, double a);  // T = a * S 
int mult_field1D(field1D *T, const field1D *S);              // T = T * S

/* field 2D function declarations */

field2D*  allocate_field2D( int , int );
void deallocate_field2D(field2D * );
int  set_field2D_to_value(field2D *T, double a);
void mult_field2D_const(field2D *T, double a);
int copy_field2D_mult(field2D *T, const field2D *S, double a);  // T = a * S 
int copy_field2D(field2D *T, const field2D *S);   // T = S 

/* Input functions declarations */

int load_field1D_ascii(char *fname, field1D *T);    /* returns 1 if successfull */
int load_field1D_binary(char *fname, field1D *T);    /* returns 1 if successfull */

int load_field2D_ascii(char *fname, field2D *T);    /* returns 1 if successfull */
int load_field2D_binary(char *fname, field2D *T);    /* returns 1 if successfull */


/* Output function declarations */


void save_field1D_ascii(char *fname, field1D *T);
void save_field1D_binary(char *fname, field1D *T);

void print_field2D(field2D *T);
void save_field2D_ascii(char *fname, field2D *T);
void save_field2D_binary(char *fname, field2D *T);


  /* generic useful function */

double max_field2D(field2D * );
double max_field1D(field1D * );
int    max_field1D_location(field1D * );
double max_array(double *arr, int num);
int    lmax_array(double *arr, int num);

#ifdef __cplusplus
}                               /* extern "C" */
#endif                          /* __cplusplus */

#endif				/* FIELD_H */
