/*
 * Copyright (c) 2002 Falk Feddersen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


/*    
      misc.c  - Misc functions that deal w/ numerical model

*/


#include <stdio.h>

#include "misc.h"


/* Here we test the various #defines that are turned on in 
   defs.h to make sure that none of them are incompatible.
   This is a garrish way of doing this but it needs to be done.
   Eventually this should be cleaned up.               */

void defs_test()
{

  /* first do the bottom boundary conditions */

#ifdef NO_FLUX_TKE_BOTTOM_BC
  #ifdef NO_SLIP_TKE_BOTTOM_BC
    fprintf(stdout,"*** Error! Two bottom boundary conditions defined!\n");
    exit(-1);
  #endif /* NO_SLIP_TKE_BOTTOM_BC */
  #ifdef MY_TKE_BOTTOM_BC
    fprintf(stdout,"*** Error! Two bottom boundary conditions defined!\n");
    exit(-1);
  #endif   /* MY_TKE_BOTTOM_BC */
#endif /* NO_FLUX_TKE_BOTTOM_BC */


 /* next do top boundary conditions */


#ifdef NO_FLUX_TKE_TOP_BC
  #ifdef SET_FLUX_TKE_TOP_BC
    fprintf(stdout,"*** Error! Two top TKE boundary conditions defined!\n");
    exit(-1);
  #endif /* SET_FLUX_TKE_TOP_BC */
#endif /* NO_FLUX_TKE_TOP_BC */


    /*
#ifdef BUBBLE_BUOYANCY_FLUX
    fprintf(stdout,"*** Error! Can't have bubble BF w/ no bubble dyamics\n");
    exit(-1);
#endif 
    */



#ifdef SEDIMENT_BUOYANCY_FLUX
    fprintf(stdout,"*** Error! Can't have bubble BF w/ no bubble dyamics\n");
    exit(-1);
#endif /* SEDIMENT_BUOYANCY_FLUX */



#ifdef CB_LENGTH_SCALE
  #ifdef WALL_LENGTH_SCALE
    fprintf(stdout,"*** Error! Two tke length scales defined!\n");
    exit(-1);
  #endif /* WALL_LENGTH_SCALE */
#endif /* CB_LENGTH_SCALEK */

#ifdef CB_LENGTH_SCALE
  #ifdef CONST_LENGTH_SCALE
    fprintf(stdout,"*** Error! Two tke length scales defined!\n");
    exit(-1);
  #endif /* CONST_LENGTH_SCALE */
#endif /* CB_LENGTH_SCALEK */

#ifdef WALL_LENGTH_SCALE
  #ifdef CONST_LENGTH_SCALE
    fprintf(stdout,"*** Error! Two tke length scales defined!\n");
    exit(-1);
  #endif /* CONST_LENGTH_SCALE */
#endif /* WALL_LENGTH_SCALE */


#ifdef WAVE_BUBBLE_SOURCE
  #ifdef SURFACE_WAVE_TKE_SOURCE
    fprintf(stdout,"*** Error! Mismatched wave & bubble tke sources.!\n");
    exit(-1);
  #endif /* SURFACE_WAVE_TKE_SOURCE */
  #ifdef SURFACE_WAVE_BUBBLE_SOURCE
    fprintf(stdout,"*** Error! Both wave bubble sources.!\n");
    exit(-1);
#endif /* SURFACE_WAVE_BUBBLE_SOURCE */
#endif /* WAVE_BUBBLE_SOURCE */

#ifdef GEOSTROPHIC_IC
  #ifndef ROTATION
    fprintf(stdout,"*** Error! geostrophic_ic specified without rotation!\n");
    exit(-1);
  #endif /* ROTATION */
#endif /* GEOSTROPHIC_IC */

#ifdef SHAW_IC
  #ifndef ROTATION
    fprintf(stdout,"*** Error! Both shaw_ic specified without rotation!\n");
    exit(-1);
  #endif /* ROTATION */
#endif /* SHAW_IC */


}

void defs_report()
{

  fprintf(stdout,"******* kepsilon %d.%d.%d : ACTIVE DEFINES *******************\n",
          MAJOR_VERSION_NUMBER,MINOR_VERSION_NUMBER,MICRO_VERSION_NUMBER);

#ifdef DEBUG
  fprintf(stdout,"kepsilon -: DEBUG is compiled in.\n");
#endif

#ifdef DEBUG2
  fprintf(stdout,"kepsilon -: DEBUG2 is compiled in.\n");
#endif

#ifndef THREAD_YES
  fprintf(stdout,"kepsilon -: THREADS and GUI *not* compiled in -> should *only* be used for debugging.\n");
#endif


#ifdef BUBBLE_BREAKING
  fprintf(stdout,"kepsilon - bubble: BUBBLE_BREAKING is compiled in.\n");
#endif

#ifdef BUBBLE_DISSOLUTION
 fprintf(stdout,"kepsilon - bubble: BUBBLE_DISSOLUTION is compiled in.\n");
#endif

#ifdef BUBBLE_BUOYANCY_FLUX
  fprintf(stdout,"kepsilon - tke: BUBBLE_BUOYANCY_FLUX is compiled in.\n");
#endif

#ifdef SEDIMENT_BUOYANCY_FLUX
  fprintf(stdout,"kepsilon - tke: SEDIMENT_BUOYANCY_FLUX is compiled in.\n");
#endif

#ifdef NO_FLUX_TKE_BOTTOM_BC
  fprintf(stdout,"kepsilon - tke: NO_FLUX_TKE_BOTTOM_BC is bottom tke BC.\n");
#endif

#ifdef MY_TKE_BOTTOM_BC
  fprintf(stdout,"kepsilon - tke: MY_TKE_BOTTOM_BC is bottom tke BC.\n");
#endif

#ifdef NO_SLIP_TKE_BOTTOM_BC
  fprintf(stdout,"kepsilon - tke: NO_SLIP_TKE_BOTTOM_BC is bottom tke BC.\n");
#endif

#ifdef CB_LENGTH_SCALE
  fprintf(stdout,"kepsilon - tke: CB_LENGTH_SCALE is tke length scale.\n");
#endif

#ifdef WALL_LENGTH_SCALE
  fprintf(stdout,"kepsilon - tke: WALL_LENGTH_SCALE is tke length scale.\n");
#endif

#ifdef CONST_LENGTH_SCALE
  fprintf(stdout,"kepsilon - tke: CONST_LENGTH_SCALE is tke length scale.\n");
#endif

#ifdef WAVE_ASYMMETRY
  fprintf(stdout,"kepsilon - wave: WAVE_ASYMMETRY is included in the wave forcing.\n");
#endif


#ifdef SHAW_TIDAL_FORCING
    fprintf(stdout,"kepsilon - mflow: SHAW_TIDAL_FORCING is compiled in.\n");
#endif

  //  if (mflow_flag) {
#ifdef NO_SLIP_MFLOW_BOTTOM_BC
    fprintf(stdout,"kepsilon - mflow: NO_SLIP_MFLOW_BOTTOM_BC is bottom mean flow BC.\n");
#endif
#ifdef NO_FLUX_MFLOW_BOTTOM_BC
    fprintf(stdout,"kepsilon - mflow: NO_FLUX_MFLOW_BOTTOM_BC is bottom mean flow BC.\n");
#endif
#ifdef ROTATION
    fprintf(stdout,"kepsilon - mflow: ROTATION dynamics are compiled in.\n");
#endif
#ifdef GEOSTROPHIC_IC
    fprintf(stdout,"kepsilon - mflow: GEOSTROPHIC_IC, mflow IC is geostrophic.\n");
#endif

    //  }


  fprintf(stdout,"******************************************************\n");
}


static const double max_S_for_stability = 100;  //!! this is totally made up for explicit!
static const double max_CFL = 0.5;

void stability_report(field1D *KV, timing *T, double dz)
{
  double max_Kv, S, CFL;
  double dt = T->dt;
  double time = dt * T->time_index;
  int max_loc;

  max_Kv = max_field1D(KV);     // get maximum vertical eddy viscosity
  max_loc = max_field1D_location(KV);

  S = max_Kv * dt / (dz*dz);    // calculate stability parameter

  if (S > max_S_for_stability) {
    fprintf(stdout,"*** Warning! Diffusive stability criteria exceeded at time=%lf at zi=%d\n",time,max_loc);
    fprintf(stdout,"    Max_Kv = %lf,  dt = %lf,  dz = %lf,  Smax = %lf\n",max_Kv,dt,dz,S);
  }

  if (S < 0) {     /* if S is negative this is REALLY bad */
    fprintf(stdout,"*** Warning! Diffusive backfire! Kv is negative at time=%lf at zi=%d\n",time,max_loc);
    fprintf(stdout,"    Max_Kv = %lf,  dt = %lf,  dz = %lf,  Smax = %lf\n",max_Kv,dt,dz,S);
    error_message("** Aborting program");
  }

  /*
  CFL = 0.3 * dt/dz;

  if (CFL > max_CFL) {
    fprintf(stdout,"*** Warning! CFL stability criteria exceeded at time=%lf!\n",time);
    fprintf(stdout,"    CFL = %lf,  dt = %lf,  dz = %lf,  CFLmax = %lf\n",CFL,dt,dz,max_CFL);
  }
  */


}


double  diffusive_stability_parameter(field1D *KV, double dt, double dz)
{
  double max_Kv;
  double S;

  max_Kv = max_field1D(KV);     // get maximum vertical eddy viscosity

  S = max_Kv * dt / (dz*dz);    // calculate stability parameter

  return S;

}



void timing_report(timing *T)
{

  fprintf(stdout,"------------- * Timing Report * --------------\n");
  fprintf(stdout,"Total Run Time = %9.3g (s)    dt = %7.3g (s)\n",T->total_run_time,T->dt);
  fprintf(stdout,"Level1 Run Time = %9.3g (s)   Level2 Run Time= %9.3g (s)\n",T->level1_time,T->level2_time);
  fprintf(stdout,"Level3 # = %d   Level2 # = %d   Level1 # = %d\n",T->level3_iter,T->level2_iter,T->level1_iter);
  fprintf(stdout,"Default time units are %s\n",T->units_ptr);
  fprintf(stdout,"-----------------------------------------------\n");
}

static char *breaking_string[3] = {"none","spilling","plunging"};

static char *smodel[3] = {"kepsilon","mellor-yamada","craig-banner"};


void wave_and_forcing_report(inputinfo *I)
{

  waveinfo *W = I->W;
  mflowinfo *M = I->M;
  bubbleinfo *B = I->B;
  sedimentinfo *S = I->SED;
  densityinfo *D = I->D;

  int nn, bb;

  switch(W->turb_source) {
  case NONE:     bb=0; break;
  case SPILLING: bb=1; break;
  case PLUNGING: bb=2; break;
  default: error_message("Bad switch arg in misc.c: wave_and_foricng_report()\n");
  }

  fprintf(stdout,"------------- * Wave, Forcing, TKE, Bubble & Sediment Report * --------------------------\n");

  /* Model type first */

  switch (I->model_type) {
  case KEPSILON:  nn = 0; break;
  case MELLOR_YAMADA: nn=1; break;
  case CRAIG_BANNER: nn=2;  break;
  }

  fprintf(stdout,"Model type = %s\n",smodel[nn]);

  /* Then model dimension information */

  fprintf(stdout,"Depth = %.1g (m),  nz=%d,  dz=%.2g (m),  z0b=%.2g (m),  z0s=%.2g (m)\n",
        I->depth,I->nz,I->dz,I->z0b,I->z0s);

  /* Bubbles Second */

  if (B->bubble_flag == ON) {
    if (B->ic_flag == ON) 
      fprintf(stdout,"Bubbles: rmin=%.4g (m), rmax=%.4g (m), nr=%d, IC:  beta0=%.3g, scale_h=%.2g (m)\n", B->rmin, B->rmax, B->nr,B->beta0,B->scale_h);
    else
      fprintf(stdout,"Bubbles: rmin=%.4g (m), rmax=%.4g (m), nr=%d,  zero IC\n", B->rmin, B->rmax, B->nr);
  }
  else
    fprintf(stdout,"Bubbles:\t module not used in this run\n");

  /* Waves Next */

  fprintf(stdout,"Depth = %4.2g (m), Hrms = %4.2g (m),  T = %4.1lf (s), theta = %3.2g\n",
	  depth,W->Hrms,W->T,W->theta_deg);
  fprintf(stdout,"k =%4.3g (1/m), kh =%4.3g, c =%2.2g (m/s), cg = %4.2g (m/s), u0 = %4.2g (m/s)\n",
                  W->wave_num,W->wave_num*depth,W->phase_v,W->group_v,W->U0);
  fprintf(stdout,"breaker_type = %s, wave diss = %.3g, dsxydx = %.3g\n",
         breaking_string[bb],W->mepsilon,W->dsxydx);

  /* Mean flow */

  if (M->mflow_flag == ON) {
    if (M->surfzone_flag == ON)
      fprintf(stdout,"Mean Flow: wind: Ustar=%.4g, Vstar=%.4g  forcing: (surfzone) Fx=%.3g Fy=%.3g\n", M->ustar, M->vstar,M->mFx,M->mFy);
    else
      fprintf(stdout,"Mean Flow: wind: Ustar=%.4g, Vstar=%.4g  forcing: Fx=%.3g Fy=%.3g\n", M->ustar, M->vstar,M->mFx,M->mFy);
  }
  else 
    fprintf(stdout,"Mean Flow:\t module not active in this run\n");

  /* --- Denisty Information --- */

  if (D->density_flag == ON) {
    switch (D->top_bc) {
    case FLUX:   fprintf(stdout,"Denisty Module: FLUX surface bc w/ value %g used - Initial condition N=%g (rad/sec).\n",
                         D->bcval,D->BVfreq);
    break;
    case FIXED:   fprintf(stdout,"Denisty Module: FIXED  surface bc w/ value %g used - Initial condition N=%g (rad/sec).\n",
                         D->bcval,D->BVfreq);
    }
  }
  else 
    fprintf(stdout,"Density:\t module not active in this run\n");

  if (S->sediment_flag == ON)
    fprintf(stdout,"Sediment:\t D50 = %.3g\n",S->D50);
  else
    fprintf(stdout,"Sediment:\t module not active in this run\n");

  fprintf(stdout,"--------------------------------------------------------------------\n");


}
