/*
 * Copyright (c) 2002 Falk Feddersen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


/* --  ouputflags.h:  defines ouput data structures: no functions */


#ifndef OUTPUTFLAGS_H
#define OUTPUTFLAGS_H


#include <stdio.h>
#include <glib.h>

#include "defs.h"


/* -------------- New Linked list stuff here --------- */


typedef enum {
  TKE_M, DISS_M, BETA_M, KQV_M, KUV_M, BFLUX_M, BUBBLE_M, U_M, V_M, Q2L_M,
  TOTTKE_M, TOTPE_M, TOTE_M, LSCALE_M, PROD_M, TKEDIFF_M, DBDT_M,
  UW_M, VW_M, PBREAKUP_M, DBREAKUP_M, BUBBLE_DISSOLUTION_M, BASE_BUBBLE_SRC_M,
  STABILITY_M, SEDIMENT_M, SEDIMENT_FLUX_U_M, SEDIMENT_FLUX_V_M, WRHO_M, WRHOFLUX_M,
  KRHOV_M
} variable_t;

typedef enum {
  ASCII_M, BINARY_M  // removed NONE type
} file_output_t;

typedef enum {
  FILE_M, GUI_M, TERM_M, NETCDF_M
} output_t;

typedef struct {
  output_t OT;
  file_output_t FOT;
  GString *fname;
  GString *message;
  FILE *fp;
  int start_time;
} output_options_t;

typedef struct {
  variable_t var;
  int pz;  // location in depth space
  int pr;  // location in radius space for bubble!  
  field1D *D;
  output_options_t *OP;
  //  Plotwindow *PW;   /* when linked to the gui */
} point;

typedef struct {
  variable_t var;
  field1D *D;
  int rloc;             // this is for BUBBLE_M output
  output_options_t *OP;
//  Plotwindow *PW;   /* when linked to the gui */
} depth_line;


typedef struct {
  variable_t var;
  field1D *D;
  output_options_t *OP;
  int zloc;
//  Plotwindow *PW;   /* when linked to the gui */
} radius_line;

typedef struct {
  variable_t var;
  field2D *D;
  output_options_t *OP;
//  Plotwindow *PW;   /* when linked to the gui */
} full_bubble_output;


typedef struct {
  GSList *point_list;
  GSList *depth_list;
  GSList *radius_list;
  full_bubble_output *Bubble_out;
} oflags1;


typedef struct {
  oflags1 *F0, *F1, *F2, *F3;
} outputflags;


FILE*  open_file_for_writing(char *s);

#endif  /* OUTPUTFLAGS_H */
