/*
 * Copyright (c) 2001 Falk Feddersen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */


/*  pinput.c -- funcs for loading init.ini file */ 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "defs.h"
#include "pinput.h"
#include "input.h"

int lineno=0;



void parse_error_message(char *s)
{

  fprintf(stderr,"Error parsing init file at line number: %d\n",lineno);
  fprintf(stderr,"%s\n",s);
  exit(-1);

}


/* --------- input file parsing function ------------- */


char* get_next_line(FILE *fp, char *s)
{
  char c1 = '%';
  char *rt = NULL;


  while (c1 == '%') {
    lineno++;
    rt=fgets(s,100,fp);
    if (rt==NULL)
      return rt;
    c1 = s[0];
  }

  return rt;
}

/* ---------- output parsing stuff -------------- */

/* ---- generic all level routinues ---- */

variable_t variable_t_index(char *a)
{

  if (!strcasecmp(a,"tke"))
    return TKE_M;

  if (!strcasecmp(a,"diss"))
    return DISS_M;

  if (!strcasecmp(a,"beta"))
    return BETA_M;

  if (!strcasecmp(a,"kqv"))
    return KQV_M;

  if (!strcasecmp(a,"kuv"))
    return KUV_M;

  if (!strcasecmp(a,"bubble"))
    return BUBBLE_M;

  if (!strcasecmp(a,"bflux"))
    return BFLUX_M;

  if (!strcasecmp(a,"v"))
    return V_M;

  if (!strcasecmp(a,"u"))
    return U_M;

  if (!strcasecmp(a,"q2l"))
    return Q2L_M;

  if (!strcasecmp(a,"tottke"))
    return TOTTKE_M;

  if (!strcasecmp(a,"totpe"))
    return TOTPE_M;

  if (!strcasecmp(a,"tote"))
    return TOTE_M;

  if (!strcasecmp(a,"tenergy"))
    return TOTE_M;

  if (!strcasecmp(a,"lscale"))
    return LSCALE_M;

  if (!strcasecmp(a,"prod"))
    return PROD_M;

  if (!strcasecmp(a,"diff"))
    return TKEDIFF_M;

  if (!strcasecmp(a,"dbdt"))
    return DBDT_M;

  if (!strcasecmp(a,"dqdt"))
    return DBDT_M;

  if (!strcasecmp(a,"uw"))
    return UW_M;

  if (!strcasecmp(a,"vw"))
    return VW_M;

  if (!strcasecmp(a,"pbreakup"))
    return PBREAKUP_M;

  if (!strcasecmp(a,"dbreakup"))
    return DBREAKUP_M;

  if (!strcasecmp(a,"bubble_dissolution"))
    return BUBBLE_DISSOLUTION_M;

  if (!strcasecmp(a,"base_bubble_src"))
    return BASE_BUBBLE_SRC_M;

  if (!strcasecmp(a,"stability"))
    return STABILITY_M;

  if (!strcasecmp(a,"sediment"))
    return SEDIMENT_M;

  if (!strcasecmp(a,"sfluxu"))
    return SEDIMENT_FLUX_U_M;

  if (!strcasecmp(a,"sediment_flux_u"))
    return SEDIMENT_FLUX_U_M;

  if (!strcasecmp(a,"sfluxv"))
    return SEDIMENT_FLUX_V_M;

  if (!strcasecmp(a,"sediment_flux_v"))
    return SEDIMENT_FLUX_V_M;

  if (!strcasecmp(a,"rho"))
    return WRHO_M;

  if (!strcasecmp(a,"density"))
    return WRHO_M;

  if (!strcasecmp(a,"wrho"))
    return WRHO_M;

  if (!strcasecmp(a,"wrhoflux"))
    return WRHOFLUX_M;

  if (!strcasecmp(a,"rhoflux"))
    return WRHOFLUX_M;

  if (!strcasecmp(a,"krhov"))
    return KRHOV_M;

  parse_error_message("Syntax Error: Bad option to variable_t_index()!");

  return TKE_M;  // this should never happen!
}


output_t output_t_index(char *s)
{

  output_t  tt;

  if (!strcasecmp(s,"file")) 
    tt = FILE_M;
  else {
    if (!strcasecmp(s,"term"))
      tt =  TERM_M;
    else 
      if (!strcasecmp(s,"gui"))
	tt =  GUI_M;
      else 
	parse_error_message("Syntax Error: Bad output  option in init file.");
  }
  
  return tt;  

}

file_output_t ascii_or_binary_new(char *s)
{

  file_output_t tt;

  if (s==NULL) {
    fprintf(stderr,"**** Using default ASCII file_output_t ***\n");
    tt =  ASCII_M;
  }

  if (!strcasecmp(s,"binary"))
    tt = BINARY_M;
  else {
    if (!strcasecmp(s,"ascii"))
      tt = ASCII_M;
    else 
      parse_error_message("Syntax Error: Bad binary/ascii option to init file.");
  }

  return tt;

}

/* Parses the common output options for all flagsn 1,2 */

output_options_t*  new_output_options()
{
  output_options_t *T;
  T = (output_options_t * ) g_malloc(sizeof(output_options_t));

  T->fp = NULL;
  T->fname = NULL;
  T->message = NULL;

  return T;
}


GString* parse_output_message(char *rest)
{
  GString *M;
  int N,i,qflag,num=0;
  char tmp[120];

  /* first get string "rest" length, then check for "" */

  N = strlen(rest);
  if (rest[0]!='"') 
    error_message("pinput.c:parse_output_message - Bad string to terminal output");

  /* now look for 2nd " */

  qflag = 0;

  for (i=1;i<N-1;i++)
    if (rest[i]=='"') {
      qflag=1;
      num = i-1;
    }

  if (qflag == 0)
    error_message("pinput.c:parse_output_message - Bad string to terminal output");

  strncpy(tmp,&(rest[1]),num);
  tmp[num] = '\0';

  /* get a new GString */

  M = g_string_new(tmp);

  return M;

}




output_options_t*  parse_output_options(char *s)
{
  output_options_t *T;
  char filet[32],abt[32],fname[32],*rest;
  char start_time[32], tunits[32];
  int num, st, un1;

  T = new_output_options(); 


  num = sscanf(s,"%s %s %s %s %s", filet, abt, fname, start_time, tunits);

  if (num==0) 
    parse_error_message("Syntax Error: No Output options given!");

  T->OT = output_t_index(filet);

  if (T->OT ==TERM_M) {
    T->fp = stdout;
    if (num!=1) {
      rest = strstr(s,abt);
      T->message = parse_output_message(rest);
    }
  }

  if (T->OT == FILE_M) {
    if ((num<3)||(num>5))
      parse_error_message("Syntax Error: Not enough file output options???");

    T->FOT = ascii_or_binary_new(abt);
    T->fname = g_string_new(fname);       
    T->fp = open_file_for_writing((T->fname)->str);


    /* This handles when the model starts writing data */

    if (num==4) {
      st = atoi(start_time);
      if (st<0)
        parse_error_message("Bad option to atoi()\n");
    }
    else {
      if (num==5) {
	st = atoi(start_time);
	if (st<0)
	  parse_error_message("Bad option to atoi()\n");
        un1 = fix_units(tunits);
        st = change_units(st,un1);
      }
      else {
	st = 0;
      }
    }

    T->start_time  = st;
  }

  return T;

}


void output_options_destroy(output_options_t *OP)
{
   if (OP->fp != NULL)
      fclose(OP->fp);

   if (OP->fname != NULL)
      g_string_free(OP->fname,TRUE);

   g_free(OP);

}

field1D*  ptr_to_field1D(variable_t T)
{

  switch (T){
  case TKE_M:     return BB;
  case Q2L_M:     return Q2L;
  case DISS_M:    return DISS;
  case KQV_M:     return KQV;
  case KUV_M:     return KUV;
  case BFLUX_M:   return BFLUX;
  case LSCALE_M:  return LL;
  case PROD_M:    return PROD;
  case TKEDIFF_M:    return TKEDIFF;
  case DBDT_M:    return BBT2;
  case UW_M:      return UW;
  case VW_M:      return VW;
  case KRHOV_M:   return KRHOV;
  case BETA_M:    
    if (bubble_flag==ON)
      return BETA;
    else
      parse_error_message("** Output Syntax Error: Bubble output not allowed if bubble dynamics are off.");

  case U_M: 
    if (mflow_flag == ON)
      return UU;
    else
      parse_error_message("** Output Syntax Error: Mean flow output not allowed if mean flow dynamics are off.");

  case V_M:    
    if (mflow_flag == ON)
      return VV;
    else
      parse_error_message("** Output Syntax Error: Mean flow output not allowed if mean flow dynamics are off.");


  case WRHO_M:
    if (density_flag==ON)
      return WRHO;
    else
      return WRHO;
    //      parse_error_message("** Output Syntax Error: Density output not allowed if density dynamics are off.");

  case WRHOFLUX_M:
    if (density_flag==ON)
      return WRHOFLUX;
    else
      parse_error_message("** Output Syntax Error: Density-flux output not allowed if density dynamics are off.");

  case SEDIMENT_M:   
    if (sediment_flag==ON)
      return SS;
    else 
      parse_error_message("** Output Syntax Error: Sediment output not allowed if sediment dynamics are off.");

  case SEDIMENT_FLUX_U_M: 
    if (sediment_flag==ON)
      return SFLUX_U;
    else 
      parse_error_message("** Output Syntax Error: Sediment output not allowed if sediment dynamics are off.");

  case SEDIMENT_FLUX_V_M: 
    if (sediment_flag==ON)
      return SFLUX_V;
    else 
      parse_error_message("** Output Syntax Error: Sediment output not allowed if sediment dynamics are off.");

  default: parse_error_message("Bad arg to ptr_to_field1D");
  }

}

int check_zloc(int a)
{

  if ((a<0)||(a>=nz)) 
    parse_error_message("Syntax Error in init file:\nBad depth index!\n");

  return a;
}


/* ---------- level 1 stuff ------------------ */

void parse_level1_point(char *s, oflags1 *F)
{
  char vtype[16],zloc[16],tmp[20],*rest;
  int num; 
  point *P;

  P = (point * ) g_malloc(sizeof(point));
  num = sscanf(s,"%*s %*s %s %s ",vtype,tmp);
  rest = strstr(s,tmp);

  if (num!=2)
    parse_error_message("Syntax Error: Not enough arguments for point output!");

  P->var = variable_t_index(vtype);

  switch (P->var) {
  case TOTE_M:
  case TOTTKE_M:
  case STABILITY_M:
  case TOTPE_M:     break;
  default:   num = sscanf(s,"%*s %*s %s %s %s",vtype,zloc,tmp);
             if (num!=3)
               parse_error_message("Syntax Error: Not enough arguments for point output!");
	     rest = strstr(s,tmp);
             P->pz = atoi(zloc);
             check_zloc(P->pz);
             P->D = ptr_to_field1D(P->var);
  }

  //  check_zloc(P->pz);   // check to make sure in bounds

  P->OP = parse_output_options(rest);

  F->point_list = g_slist_append(F->point_list,P);

}


void parse_level1_depth(char *s, oflags1 *F)
{
  char vtype[16], tmp[80], *rest;
  int num, rloc;
  depth_line *P;

  P = (depth_line * ) g_malloc(sizeof(depth_line));

  num = sscanf(s,"%*s %*s %s %s",vtype,tmp);
  rest = strstr(s,tmp);

  if (num!=2)
    parse_error_message("Syntax Error: Not enough arguments for depth output!");


  P->var = variable_t_index(vtype);


  if (P->var == BUBBLE_M) {
    num = sscanf(s,"%*s %*s %s %d %s",vtype,&rloc,tmp);
    rest = strstr(s,tmp);
    if (num!=3)
      parse_error_message("Syntax Error: Not enough arguments for depth output!");
    P->rloc = rloc;
  }
  else
    P->D = ptr_to_field1D(P->var);

  P->OP = parse_output_options(rest);

  F->depth_list = g_slist_append(F->depth_list,P);

}


field1D* ptr_to_radius(variable_t T, int zlevel)
{
  field1D *D;

  D = allocate_field1D(0);
  D->N = NB->M;

  check_zloc(zlevel);

  switch (T) {
  case BUBBLE_M:   D->data = NB->row[zlevel];
    break;
  case PBREAKUP_M:  D->data = GN->row[zlevel];
    break;
  case DBREAKUP_M:  D->data = BR->row[zlevel];
    break;
  case BUBBLE_DISSOLUTION_M:  D->data = BDS->row[zlevel];
    break;
  case BASE_BUBBLE_SRC_M:  D->data = BASE_NN_SOURCE->row[zlevel];
    break;
  default: parse_error_message("Syntax Error: expected bubble type for radius output");
  }

  return D;

}


void parse_level1_radius(char *s, oflags1 *F)
{
  char vtype[16], tmp[80],*rest;
  int num, zloc; 
  radius_line *P;

  P = (radius_line * ) g_malloc(sizeof(radius_line));

  num = sscanf(s,"%*s %*s %s %d %s",vtype,&zloc,tmp);
  rest = strstr(s,tmp);

  if (num!=3)
    parse_error_message("Syntax Error: Not enough arguments for depth output!");

  P->var = variable_t_index(vtype);

  P->D = ptr_to_radius(P->var,zloc);
  P->zloc = zloc;

  P->OP = parse_output_options(rest);

  F->radius_list = g_slist_append(F->radius_list,P);

}


void parse_level1(char *s, oflags1 *F)
{
  char type[16], rest[80];
  int num;

  num = sscanf(s,"%*s %s %s",type,rest);
  lineno++;

  if (num != 2)
    parse_error_message("Syntax Error: Not enough arguments for level1 output!");

  if (!strcasecmp(type,"point")) {
    parse_level1_point(s,F);
    return;
  }

  if (!strcasecmp(type,"depth")) {
    parse_level1_depth(s,F);
    return;
  }

  if (!strcasecmp(type,"radius")) {
    parse_level1_radius(s,F);
    return;
  }

  parse_error_message("Syntax Error: Wrong type of level1 output.  Need [point,depth,radius].");

}


oflags1* oflags1_new()
{
  oflags1 *F;

  F = (oflags1 * ) g_malloc(sizeof(oflags1));

  F->point_list = NULL;
  F->depth_list = NULL;
  F->radius_list = NULL;
  F->Bubble_out = NULL;

  return F;
}



void point_destroy(point *P)
{

   output_options_destroy(P->OP);
   g_free(P);

}

void depth_line_destroy(depth_line *P)
{

   output_options_destroy(P->OP);
   g_free(P);

}

void radius_line_destroy(radius_line *P)
{

   output_options_destroy(P->OP);
   g_free(P);

}



void oflags1_destroy(oflags1 *F)
{

  GSList *list;

  for (list=F->point_list;list;list=list->next)
     point_destroy(list->data);

  for (list=F->depth_list;list;list=list->next)
     depth_line_destroy(list->data);

  for (list=F->radius_list;list;list=list->next)
     radius_line_destroy(list->data);

  g_slist_free(F->point_list);
  g_slist_free(F->depth_list);
  g_slist_free(F->radius_list);

  if (F->Bubble_out)
    g_free(F->Bubble_out);

}



void outputflags_destroy(outputflags *A)
{
  if (A!=NULL) {
    oflags1_destroy(A->F0);
    oflags1_destroy(A->F1);
    oflags1_destroy(A->F2);
    oflags1_destroy(A->F3);
  }

}



/* ------- all parsing stuff stuff ------ */

void parse_line(char *s,outputflags *A)
{
  char level[16];
  int num;

  num = sscanf(s,"%s %*s",level);

  if (num!=1) {
    parse_error_message("Error: Could not assign string in parse_line!\n");
    fprintf(stderr,"Syntax error in init file.\n");
    exit(-1);
  }

  if (!strcasecmp(level,"level1")) 
    parse_level1(s,A->F1);
  else {
   if (!strcasecmp(level,"level2")) {
     parse_level1(s,A->F2);
    }
   else {
     if (!strcasecmp(level,"level3")) {
       parse_level1(s,A->F3);
     }
     else {
       if (!strcasecmp(level,"level0")) {
	 parse_level1(s,A->F0);
       }
       else {
	 parse_error_message("Syntax Error in the init file!\n");
	 exit(-1);
       }
     }
   }
  }
  
}


outputflags* parse_output(FILE *fin)
{

  oflags1 *F0, *F1, *F2,*F3;
  outputflags *A;

  char s[120];

  F0 = oflags1_new();
  F1 = oflags1_new();   // initialize the flag values
  F2 = oflags1_new();
  F3 = oflags1_new();

  A = (outputflags * ) g_malloc(sizeof(outputflags));
  A->F0 = F0;
  A->F1 = F1;
  A->F2 = F2;
  A->F3 = F3;

  while (get_next_line(fin, s)!=NULL) 
    if (strlen(s)>1) {
      parse_line(s,A);   /* BUGS for level3 ********* */
#ifdef DEBUG
      g_print("Line: %s parsed\n",s);
#endif
    }

  return A;

}



void parse_model_dimensions(char *s, inputinfo *I)
{

  int aa;
  double  bb;
  int num;

  double z0b, z0s;   // bed and surface length scales

  num = sscanf(s,"%d  %lf %lf %lf",&aa, &bb,&z0b, &z0s);

  if ((num<2) || (num>4))
    parse_error_message("Wrong number of parameters given: either 2,3,or 4");

  I->nz = aa;  
  I->dz = bb;
  I->depth = (aa-1)*bb;

  switch (num) {
  case 2:  I->z0b = 0.001;   // default 1mm bed roughness
           I->z0s = 0.2;     // default 20cm suface roughness
           break;
  case 3:  I->z0b = z0b;
           I->z0s = 0.2;     // default 20cm suface roughness
           break;
  case 4:  I->z0b = z0b;
           I->z0s = z0s;     // default 20cm suface roughness
           break;
  default: parse_error_message("!!Prog error in parse_model_dimensions()");
  }


}


void parse_model_type(char *s, inputinfo *I)
{

  int tke_ic;
  char bub[120], bic_file[120], tkebc[120];

  sscanf(s,"%s %d  %s %s",bub,&tke_ic,bic_file,tkebc);    

  if (!strcasecmp(bub,"kepsilon")) {
    I->model_type = KEPSILON;
    return;
  }

  if (!strcasecmp(bub,"my")) {
    I->model_type = MELLOR_YAMADA;
    return;
  }

  if (!strcasecmp(bub,"mellor")) {
    I->model_type = MELLOR_YAMADA;
    return;
  }

  if (!strcasecmp(bub,"mellor-yamada")) {
    I->model_type = MELLOR_YAMADA;
    return;
  }

  if (!strcasecmp(bub,"mellor_yamada")) {
    I->model_type = MELLOR_YAMADA;
    return;
  }

  if (!strcasecmp(bub,"CB")) {
    I->model_type = CRAIG_BANNER;
    return;
  }

  if (!strcasecmp(bub,"craig")) {
    I->model_type = CRAIG_BANNER;
    return;
  }

  if (!strcasecmp(bub,"craig-banner")) {
    I->model_type = CRAIG_BANNER;
    return;
  }

  if (!strcasecmp(bub,"craig_banner")) {
    I->model_type = CRAIG_BANNER;
    return;
  }

  parse_error_message("Bad type of tke model chose");


}

void parse_tke_info(char *s, inputinfo *I)
{

  int tke_ic;
  char bub[120], bic_file[120], tkebc[120];

  sscanf(s,"%s %d  %s %s",bub,&tke_ic,bic_file,tkebc);    

  if (strcasecmp(bub,"tke")) 
    parse_error_message("Bad first entry for tke input");
  

  I->tke_ic = tke_ic;   
  if (tke_ic<0 || tke_ic > 1)
    parse_error_message("** Bad tke IC flag in init file");

  I->tke_ic_file = g_string_new(bic_file);
  if (!strcasecmp(tkebc,"slip"))
    I->tke_bottom_bc = SLIP;
  else {
     if (!strcasecmp(tkebc,"noslip")) {
       I->tke_bottom_bc = NOSLIP;
     }
     else 
       parse_error_message("** Bad tke bottom bc specified in init file\n");
  }

}


void parse_bubble_info(char *s, inputinfo *I)
{
  bubbleinfo *B;
  //  int bbic;
  double beta0, scale_h;
  char bub[120];
  char tmp[120],stmp[120],icstr[120];
  double rmin, rmax;
  int nr, num;

  /* Here's what it can be .....
     'bubble [on,off] rmin rmax nr  (ic  beta0  scale_h)'  */

  num=sscanf(s,"%s %s %lf  %lf %d  %119c",bub,tmp,&rmin,&rmax,&nr,stmp);

  if (strcasecmp(bub,"bubble")) {
    parse_error_message("** Syntax Error: Bad first argument for bubble input line");
  }

  if ((num<2)||(num>6))
    parse_error_message("** Syntax Error: Bad number of arguments for bubble input line");

  B = (bubbleinfo *) g_malloc(sizeof(bubbleinfo));
  B->scale_h = 0;

  if (!strcasecmp(tmp,"on")) {
    if (num>4) 
      B->bubble_flag = ON;
    else
      parse_error_message("** Syntax Error: Missing radius information arguments on bubble input line");
  }
  else {
    if (!strcasecmp(tmp,"off")) {
      B->bubble_flag = OFF;
      B->nr = 5;
      B->beta0 = 0;
      B->scale_h = 1;
      I->B = B;
      return;
    }
    else
      parse_error_message("** Syntax Error: Bad 2nd option for bubble input line");
  }

  B->rmin = rmin;
  B->rmax = rmax;
  B->nr = nr;

  /*  Now do the optional IC processing */

  B->beta0 = 0;        // this is setup
  B->scale_h = 1000.0;
  B->ic_flag = OFF;   // defalt is no IC

  I->B = B;

  if (num==5)          // then return
    return;

  /* If num>5, then there is IC processing to do!! */

  /* first make stmp have a zero at the end */

  stmp[119] = '\0';

  if (stmp[0]=='%') return;   /* and check for a comment */

  num = sscanf(stmp,"%s %lf %lf",icstr,&beta0,&scale_h);

  if (num!=3)
    parse_error_message("** Syntax Error: Wrong number of  optional bubble IC options");

  if (strcasecmp(icstr,"ic")) {
    parse_error_message("** Syntax Error: Need 'ic' option after bubble radii specifications");
  }

  B->beta0 = beta0;
  B->scale_h = scale_h;

  return;

}

void parse_wave_info(char *s, inputinfo *I)
{

  double hrms, T, theta, epsilon;
  char bub[120],dog[120],dog2[120];
  static char *serr = "Bad wave input.\n Usage:  'wave  (wave_forcing)[on,off] (turb source)[on,off] Hrms (m) T (sec)  theta (deg) [wave epsilon (m^3/s^3)'";
  waveinfo *W;
  int num;

  num = sscanf(s,"%s %s %s %lf %lf %lf %lf",bub,dog,dog2, &hrms, &T, &theta, &epsilon);

  if ((num<3)||(num>7))
    parse_error_message(serr);

  if (strcasecmp(bub,"wave")) {
    parse_error_message(serr);
  }

  W = (waveinfo * ) g_malloc(sizeof(waveinfo));

  if (!strcasecmp(dog,"on")) 
    W->wave_forcing = ON;
  else {
    if (!strcasecmp(dog,"off")) 
      W->wave_forcing = OFF;
    else {
      parse_error_message(serr);
    }
  }

  if (!strcasecmp(dog2,"plunging")) 
    W->turb_source = PLUNGING;
  else {
    if (!strcasecmp(dog2,"spilling"))
      W->turb_source = SPILLING;
    else {
      if (!strcasecmp(dog2,"off"))
        W->turb_source = NONE;
      else
        parse_error_message(serr);
    }
  }

  /* If no turb or wave forcing set stuff to a default and return */

  if ((W->wave_forcing==OFF)&&(W->turb_source==NONE)) {
    W->Hrms=0.1;
    W->T = 100;
    W->theta_deg = 0;
    W->theta = theta*pi/180.0;
    W->wepsilon_specified = -1;
    I->W = W;
    return;
  }

  if (num<6)   
    parse_error_message(serr);

  W->Hrms = hrms;
  W->T = T;
  W->theta = theta*pi/180.0;
  W->theta_deg = theta;
  W->wepsilon_specified = -1;
  W->args = 6;

  if (num==7) {
    W->wepsilon_specified = epsilon;
    W->args = 7;
  }

  I->W = W;

}

void parse_mflow_info(char *s, inputinfo *I)
{

  double ustar, vstar, Fx=0.0, Fy=0.0; 
  char bub[120];
  char mflowf[120];
  char tmp[120];
  char stmp[120];
  //  char *rest;
  int num;
  mflowinfo *M;

  num = sscanf(s,"%s %s %lf %lf %119c",bub,mflowf,&ustar,&vstar,tmp);
  tmp[119] = '\0';

  if (strcasecmp(bub,"mflow")) {
    parse_error_message("Bad first line for mean flow input");
  }

  //printf("mflow NUM = %d\n",num);
  if (num<4)
    parse_error_message("Not enough inputs to mean flow input.");

  M = (mflowinfo * ) g_malloc(sizeof(mflowinfo));



  if (!strcasecmp(mflowf,"on"))
    M->mflow_flag = ON;
  else {
    if (!strcasecmp(mflowf,"off"))
      M->mflow_flag = OFF;
    else 
      parse_error_message("Mean flow must be set either [on,off].");
  }

  M->ustar = ustar;
  M->vstar = vstar;
  M->mFx = 0.0;
  M->mFy = 0.0;
  M->surfzone_flag = OFF;

  I->M = M;

  if (num==4)
    return;

  if (tmp[0]=='%')   /* this is for a comment after 4 inputs */
    return;

  //  rest = strstr(s,tmp);

  num = sscanf(tmp,"%s",stmp);

  if (!strcasecmp(stmp,"surfzone")) {
    M->surfzone_flag = ON;
    return;
  }
  else {
    M-> surfzone_flag = OFF;
    num = sscanf(tmp,"%lf %lf",&Fx,&Fy);
    if (num!=2)
      parse_error_message("Bad mean forcing inputs to mean flow input.");
    M->mFx = Fx;
    M->mFy = Fy;
  }

  I->M = M;
  //  printf("** M->mFx=%lf  M->mFy=%lf  fx=%lf  fy=%lf\n",M->mFx,M->mFy,Fx,Fy);
}

/* Parses the denisty information for the model.   The form of this line is
   'density [on,off]  ([flux,fixed] val)  ... need something for light dIdz */

void parse_density_info(char *s, inputinfo *I)
{

  double bcvalue; 
  double NN;       // the value of the Brunt-Vaisala freq.
  char bub[120];
  char densityf[120];
  char topbc[120];      // this should either be 
  int num;
  densityinfo *D;

  num = sscanf(s,"%s %s %s %lf %lf",bub,densityf,topbc,&bcvalue,&NN);


  if (strcasecmp(bub,"density")) {
    parse_error_message("Bad first line for density  input");
  }

  if ((num<2)||(num>5))
    parse_error_message("Not enough inputs for density input.");

  D = (densityinfo * ) g_malloc(sizeof(densityinfo));
  D->BVfreq = 0.0;

  if (!strcasecmp(densityf,"on"))
    D->density_flag = ON;
  else {
    if (!strcasecmp(densityf,"off")) {
      D->density_flag = OFF;
      I->D = D;
      return;
    }
    else 
      parse_error_message("density must be set either [on,off].");
  }

  if (!strcasecmp(topbc,"flux"))
      D->top_bc = FLUX;
  else {
    if (!strcasecmp(topbc,"fixed"))
      D->top_bc = FIXED;
    else {
      printf("%s\n",topbc);
      parse_error_message("bad bc for density.  Use either 'flux' or 'fixed'.");
    }
  }

  D->bcval = bcvalue;
  D->BVfreq = NN;           // this is the B-V freq;
  I->D = D;

}


void parse_sediment_info(char *s, inputinfo *I)
{

  double D50; 
  char bub[120];
  char sedimentf[120];
  int num;
  sedimentinfo *SED;

  num = sscanf(s,"%s %s %lf",bub,sedimentf,&D50);

  if (strcasecmp(bub,"sediment")) {
    parse_error_message("Bad first line for sediment input");
  }

  if ((num<2)||(num>3))
    parse_error_message("Not enough inputs for sediment input.");

  SED = (sedimentinfo * ) g_malloc(sizeof(sedimentinfo));

  if (!strcasecmp(sedimentf,"on"))
    SED->sediment_flag = ON;
  else {
    if (!strcasecmp(sedimentf,"off")) {
      SED->sediment_flag = OFF;
      I->SED = SED;
      return;
    }
    else 
      parse_error_message("sediment must be set either [on,off].");
  }

  if (num!=3)
    parse_error_message("sediment module must have grain size (D50) specified");

  SED->D50 = D50;

  I->SED = SED;

}


inputinfo* load_init_file(GString *fname)
{

     FILE *finit;
     char s[120];
     //     gdouble a,b,c,d;
     int aa;
     gdouble bb, z0b;
    
     inputinfo *I;


     if ((finit=fopen(fname->str,"r"))==0) {
       fprintf(stderr,"Error opening  init file: %s \n",fname->str);
       exit(-1);
     }

     /* Allocate all the memory needed */

     I = (inputinfo * ) g_malloc(sizeof(inputinfo));

     /* get the base model dimension (nz,dz, etc)  */

     get_next_line(finit,s);

     parse_model_dimensions(s, I);

     /* get the type of model */

     get_next_line(finit,s);
     parse_model_type(s, I);

     /* get the TKE IC's strings */ 

     get_next_line(finit,s);
     parse_tke_info(s, I);
         
     /* get the BUBBLE IC's */ 

     get_next_line(finit,s);
     parse_bubble_info(s, I);

     /* get the Wave IC's */

     get_next_line(finit,s);
     parse_wave_info(s, I);

     /* get the Mean Flow IC's */

     get_next_line(finit,s);
     parse_mflow_info(s, I);

     /* get the Density information */

     get_next_line(finit,s);
     parse_density_info(s, I);

     /* get the Sediment information */
     get_next_line(finit,s);
     parse_sediment_info(s, I);

     /* load the timing information */

     get_next_line(finit,s);
     I->T = process_timing(s);

     /* process the init file - needs to be done before parsing output */

     process_init_file(I);
  
     /* parse the output routines */
     /* this should really be outside of this part */

     I->A = parse_output(finit);

     /* close the input file */

     fclose(finit);

     return I;
}



/* ------- init file processing--------------

   This is run AFTER the init file is read.  Here
   memory is allocated and initialized and static variables
   are initialized and that sort of thing.  Preparing to
   begin the integration   

*/

void initialize_all_variables(inputinfo *I);

void process_init_file(inputinfo *I)
{

  initialize_all_variables(I);

}
