/*
 * Copyright (c) 2002 Falk Feddersen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */



/* time_stepping.c   for kepsilon model */
     

#include "defs.h"
#include "time_stepping.h"
#include "tke.h"
#include "bubble.h"
#include "mflow.h"
#include "wave.h"
#include "sediment.h"
#include "implicit.h"
#include "density.h"
#include "q2l.h"      // here is the MY2.5 code




/* Here is the time stepping function */

void time_stepAB2_CB(timing *T)
{

   double dt;
   double bbed;

   dt = T->dt;

   /* first time step */

   calculate_tke_stats(QQ, DISS, KQV, KUV, BB, LL);


   if (density_flag == ON) {
     calculate_density_stats(DIDZ1, KRHOV, KUV);
     implicit_solve_densityAB2(WRHO, DIDZ1, DIDZ2, KRHOV, dt);
   }

   if (bubble_flag==ON) 
     calculate_bubble_stats(DISS);

   calculate_wave_stats(T->time_index, dt);

   if (sediment_flag==ON) {
     calculate_sediment_stats(KSV);
     rhs_implicit_sediment_eq(ST1,SS,KSV);
   }

   if (bubble_flag==ON)
     rhs_implicit_bubble_eq(NBT1,NB);

   rhs_implicit_tke_eq(BBT1,BB,QQ,DISS,KQV,&bbed);

   if (mflow_flag==ON) 
     rhs_implicit_mflow_eq(UT1,VT1,UU,VV);

   implicit_solve_tkeAB2(BB,BBT1,BBT2,KQV,bbed,dt);

   if (bubble_flag==ON) {
     implicit_solve_bubbleAB2(NB,NBT1,NBT2,KQV,dt);
   }


   if (mflow_flag==ON) {
     implicit_solve_mflowAB2(UU,UT1,UT2,VV,VT1,VT2,KUV,dt);
   }

   if (sediment_flag==ON) {
     implicit_solve_sedimentAB2(SS,ST1,ST2,KSV,dt);
   }

   T->time_index++;
   T->current_time += dt;

 /*--------  second time step ------------- */

   calculate_tke_stats(QQ, DISS, KQV, KUV, BB, LL);

   if (density_flag == ON) {
     calculate_density_stats(DIDZ1, KRHOV, KUV);
     implicit_solve_densityAB2(WRHO, DIDZ2, DIDZ1, KRHOV, dt);
   }

   if (bubble_flag==ON) 
     calculate_bubble_stats(DISS);

   calculate_wave_stats(T->time_index, dt);

   if (sediment_flag==ON) {
     calculate_sediment_stats(KSV);
     rhs_implicit_sediment_eq(ST2,SS,KSV);
   }

   if (bubble_flag==ON) 
     rhs_implicit_bubble_eq(NBT2,NB);

   rhs_implicit_tke_eq(BBT2,BB,QQ,DISS,KQV,&bbed);
   if (mflow_flag==ON) 
     rhs_implicit_mflow_eq(UT2,VT2,UU,VV);

   implicit_solve_tkeAB2(BB,BBT2,BBT1,KQV,bbed,dt);

   if (bubble_flag==ON) {
     implicit_solve_bubbleAB2(NB,NBT2,NBT1,KQV,dt);
   }

   if (mflow_flag==ON) {
     implicit_solve_mflowAB2(UU,UT2,UT1,VV,VT2,VT1,KUV,dt);
   }

   if (sediment_flag==ON) {
     implicit_solve_sedimentAB2(SS,ST2,ST1,KSV,dt);
   }

   /* update the time */

   T->time_index++;
   T->current_time += dt;
}



void time_step_zero_CB(timing *T)
{

   double dt;
   double bbed;

   dt = T->dt;

   /* time step zero */

   calculate_tke_stats(QQ, DISS, KQV, KUV, BB, LL);

   if (density_flag == ON) {
     calculate_density_stats(DIDZ1,KRHOV, KUV);
     implicit_solve_density_euler(WRHO, DIDZ2, KRHOV, dt);
   }

   if (bubble_flag==ON) 
     calculate_bubble_stats(DISS);

   calculate_wave_stats(T->time_index,dt);

   if (sediment_flag==ON) {
     calculate_sediment_stats(KSV);
     rhs_implicit_sediment_eq(ST2,SS,KSV);
   }

   if (bubble_flag==ON) 
     rhs_implicit_bubble_eq(NBT2,NB);

   rhs_implicit_tke_eq(BBT2,BB,QQ,DISS,KQV,&bbed);
   if (mflow_flag==ON) 
     rhs_implicit_mflow_eq(UT2,VT2,UU,VV);

   implicit_solve_tke_euler(BB,BBT2,KQV,bbed,dt);

   if (bubble_flag==ON) {
     implicit_solve_bubble_euler(NB,NBT2,KQV,dt);
   }


   if (mflow_flag==ON) {
     implicit_solve_mflow_euler(UU,UT2,VV,VT2,KUV,dt);
   }

   if (sediment_flag==ON) {
     implicit_solve_sediment_euler(SS,ST2,KSV,dt);
   }

   /* update the time */

   T->time_index++;
   T->current_time += dt;

}



/* ********************************************** */
/*  k - epsilon time stepping routinues           */
/* ********************************************   */


void time_stepAB2_kepsilon(timing *T)
{

   double dt;
   double kbed, dissbed;
   dt = T->dt;

   /* first time step */

   calculate_kepsilon_stats(BB, DISS, KQV, KUV);

   if (density_flag == ON) {
     calculate_density_stats(DIDZ1,KRHOV, KUV);
     implicit_solve_densityAB2(WRHO, DIDZ1, DIDZ2, KRHOV, dt);
   }

   if (bubble_flag==ON) 
     calculate_bubble_stats(DISS);

   calculate_wave_stats(T->time_index, dt);

   if (sediment_flag == ON) {
     calculate_sediment_stats(KSV);
     rhs_implicit_sediment_eq(ST1,SS,KSV);
   }

   if (bubble_flag==ON)
     rhs_implicit_bubble_eq(NBT1,NB);

   rhs_implicit_kepsilon_eq(BBT1,DDT1,BB,DISS,PROD,KUV,&kbed,&dissbed);

   if (mflow_flag==ON) 
     rhs_implicit_mflow_eq(UT1,VT1,UU,VV);

   implicit_solve_kepsilonAB2(BB,BBT1,BBT2,DISS,DDT1,DDT2,KUV,kbed,dissbed,dt);

   if (bubble_flag==ON) {
     implicit_solve_bubbleAB2(NB,NBT1,NBT2,KQV,dt);
   }

   if (mflow_flag==ON) {
     implicit_solve_mflowAB2(UU,UT1,UT2,VV,VT1,VT2,KUV,dt);
   }

   if (sediment_flag==ON) {
     implicit_solve_sedimentAB2(SS,ST1,ST2,KSV,dt);
   }


   T->time_index++;
   T->current_time += dt;

   /* second time step */

   calculate_kepsilon_stats(BB, DISS, KQV, KUV);

   if (density_flag == ON) {
     calculate_density_stats(DIDZ1,KRHOV, KUV);
     implicit_solve_densityAB2(WRHO, DIDZ2, DIDZ1, KRHOV, dt);
   }


   if (bubble_flag==ON) 
     calculate_bubble_stats(DISS);

   calculate_wave_stats(T->time_index, dt);

   if (sediment_flag==ON) {
     calculate_sediment_stats(KSV);
     rhs_implicit_sediment_eq(ST2,SS,KSV);
   }

   if (bubble_flag==ON) 
     rhs_implicit_bubble_eq(NBT2,NB);

   rhs_implicit_kepsilon_eq(BBT2, DDT2,BB,DISS,PROD,KUV,&kbed,&dissbed);

   if (mflow_flag==ON) 
     rhs_implicit_mflow_eq(UT2,VT2,UU,VV);

   implicit_solve_kepsilonAB2(BB,BBT2,BBT1,DISS,DDT2,DDT1,KUV,kbed,dissbed,dt);

   if (bubble_flag==ON) {
     implicit_solve_bubbleAB2(NB,NBT2,NBT1,KQV,dt);
   }

   if (mflow_flag==ON) {
     implicit_solve_mflowAB2(UU,UT2,UT1,VV,VT2,VT1,KUV,dt);
   }

   if (sediment_flag==ON) {
     implicit_solve_sedimentAB2(SS,ST2,ST1,KSV,dt);
   }

   /* update the time */

   T->time_index++;
   T->current_time += dt;
}



void time_step_zero_kepsilon(timing *T)
{

   double dt;
   double kbed, dissbed;

   dt = T->dt;

   /* time step zero */

   calculate_kepsilon_stats(BB, DISS, KQV, KUV);

   if (density_flag==ON) {
     calculate_density_stats(DIDZ1, KRHOV, KUV);
     implicit_solve_density_euler(WRHO, DIDZ2, KRHOV, dt);
   }


   if (bubble_flag==ON) 
     calculate_bubble_stats(DISS);

   calculate_wave_stats(T->time_index,dt);

   if (sediment_flag==ON) {
     calculate_sediment_stats(KSV);
     rhs_implicit_sediment_eq(ST2,SS,KSV);
   }

   if (bubble_flag==ON) 
     rhs_implicit_bubble_eq(NBT2,NB);


   rhs_implicit_kepsilon_eq(BBT2, DDT2,BB,DISS,PROD,KUV,&kbed,&dissbed);

   if (mflow_flag==ON) 
     rhs_implicit_mflow_eq(UT2,VT2,UU,VV);

   implicit_solve_kepsilon_euler(BB,BBT2,DISS,DDT2,KUV,kbed,dissbed,dt);


   if (bubble_flag==ON) {
     implicit_solve_bubble_euler(NB,NBT2,KQV,dt);
   }


   if (mflow_flag==ON) {
     implicit_solve_mflow_euler(UU,UT2,VV,VT2,KUV,dt);
   }

   if (sediment_flag==ON) {
     implicit_solve_sediment_euler(SS,ST2,KSV,dt);
   }

   /* update the time */

   T->time_index++;
   T->current_time += dt;

}


/* ********************************************** */
/*  q^2 - q^2l  time stepping routinues           */
/* ********************************************   */


void time_stepAB2_q2l(timing *T)
{

   double dt;

   dt = T->dt;



   /* first time step */

   calculate_q2l_stats(QQ, DISS, KQV, KUV, BB, Q2L);

   if (bubble_flag==ON) 
     calculate_bubble_stats(DISS);

   calculate_wave_stats(T->time_index, dt);

   if (sediment_flag==ON) {
     calculate_sediment_stats(KSV);
     rhs_implicit_sediment_eq(ST1,SS,KSV);
   }

   /* first do the BC just for kicks */

   if (bubble_flag==ON)
     rhs_implicit_bubble_eq(NBT1,NB);


   rhs_q2l_eq(BBT1, Q2LT1, BB, Q2L, DISS, PROD, KQV);

   if (mflow_flag==ON) 
     rhs_implicit_mflow_eq(UT1,VT1,UU,VV);

   update_q2l_AB2(BB,BBT1,BBT2,Q2L,Q2LT1,Q2LT2,dt);
   update_q2l_bc(BB,Q2L,KUV);


   if (bubble_flag==ON) {
     implicit_solve_bubbleAB2(NB,NBT1,NBT2,KQV,dt);
   }

   if (mflow_flag==ON) {
     implicit_solve_mflowAB2(UU,UT1,UT2,VV,VT1,VT2,KUV,dt);
   }

   if (sediment_flag==ON) {
     implicit_solve_sedimentAB2(SS,ST1,ST2,KSV,dt);
   }

   T->time_index++;
   T->current_time += dt;

   /* second time step */

   calculate_q2l_stats(QQ, DISS, KQV, KUV, BB, Q2L);

   if (bubble_flag==ON) 
     calculate_bubble_stats(DISS);

   calculate_wave_stats(T->time_index, dt);

   if (sediment_flag==ON) {
     calculate_sediment_stats(KSV);
     rhs_implicit_sediment_eq(ST2,SS,KSV);
   }


   if (bubble_flag==ON) 
     rhs_implicit_bubble_eq(NBT2,NB);

   rhs_q2l_eq(BBT2, Q2LT2, BB, Q2L, DISS, PROD, KQV);

   if (mflow_flag==ON) 
     rhs_implicit_mflow_eq(UT2,VT2,UU,VV);

   update_q2l_AB2(BB,BBT2,BBT1,Q2L,Q2LT2,Q2LT1,dt);
   update_q2l_bc(BB,Q2L,KUV);

   if (bubble_flag==ON) {
     implicit_solve_bubbleAB2(NB,NBT2,NBT1,KQV,dt);
   }

   if (mflow_flag==ON) {
     implicit_solve_mflowAB2(UU,UT2,UT1,VV,VT2,VT1,KUV,dt);
   }

   if (sediment_flag==ON) {
     implicit_solve_sedimentAB2(SS,ST2,ST1,KSV,dt);
   }

   /* update the time */

   T->time_index++;
   T->current_time += dt;
}



void time_step_zero_q2l(timing *T)
{

   double dt;

   dt = T->dt;

   /* time step zero */

   calculate_q2l_stats(QQ, DISS, KQV, KUV, BB, Q2L);

   if (bubble_flag==ON) 
     calculate_bubble_stats(DISS);

   calculate_wave_stats(T->time_index,dt);

   if (sediment_flag==ON) {
     calculate_sediment_stats(KSV);
     rhs_implicit_sediment_eq(ST2,SS,KSV);
   }

   if (bubble_flag==ON) 
     rhs_implicit_bubble_eq(NBT2,NB);

   rhs_q2l_eq(BBT2, Q2LT2, BB, Q2L, DISS, PROD, KQV);

   if (mflow_flag==ON) 
     rhs_implicit_mflow_eq(UT2,VT2,UU,VV);

   update_q2l_euler(BB,BBT2,Q2L,Q2LT2,dt);
   update_q2l_bc(BB,Q2L,KUV);

   if (bubble_flag==ON) {
     implicit_solve_bubble_euler(NB,NBT2,KQV,dt);
   }

   if (mflow_flag==ON) {
     implicit_solve_mflow_euler(UU,UT2,VV,VT2,KUV,dt);
   }

   if (sediment_flag==ON) {
     implicit_solve_sediment_euler(SS,ST2,KSV,dt);
   }

   /* update the time */

   T->time_index++;
   T->current_time += dt;

}
