/*
 * Copyright (c) 2002 Falk Feddersen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/* tke.h --- routines for the TKE dynamics */


void deallocate_tke_fields();

void init_tke_fields(int nz, double z0b, double l);

/* --- Fixed length scale Mellor-Yamada functions --- */

void rhs_implicit_tke_eq(field1D *BT, const field1D *B, const field1D *Q, const field1D *DISS, 
                         const field1D *Kv, double *bbed);

//void update_MY_tke_bc(field1D *BB, field1D *Kv);

void calculate_tke_stats(field1D *Q, field1D *DISS, field1D *Kv, field1D *KUv, const field1D *B, const field1D *LL);

void calculate_tke_stats_new(field1D *Q, field1D *DISS, field1D *Kv, field1D *KUv, const field1D *B, const field1D *LL);

void set_const_Kv(field1D *Kv, double kv);

void set_CB_length_scale(field1D *LL, double z0b, double z0top);   // sets a Craig&Banner length scale 

void set_wall_length_scale(field1D *LL, double z0b);               // sets a linear length scale

void set_parabola_length_scale(field1D *LL, double z0b, double z0top); // parabola length scale from Burchard & GOTM

/* --- k - epsilon functions --- */

void rhs_kepsilon_eq(field1D *BT, field1D *DT, const field1D *B, const field1D *DISS, const field1D *PROD, const field1D *Kv);

void rhs_implicit_kepsilon_eq(field1D *BT, field1D *DT, const field1D *B, const field1D *DISS, 
                              const field1D *PROD, const field1D *Kv, double *kbed, double *dissbed);

void update_kepsilon_bc(field1D *BB, field1D *DISS, const field1D *Kv);

void calculate_kepsilon_stats(field1D *BB, field1D *DISS, field1D *Kv, field1D *KUv);

void check_dissipation_stratified(field1D *DISS, field1D *N2);

