/*
 * Copyright (c) 2002 Falk Feddersen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/* 
   wave.h - definitions header file for wave.c

   Here all the wave stuff is done.   Changing the U(z),
   determining if wave breaking is occuring, and the 
   delta function tke & bubble sources 



*/

/* this is a test comment */

#ifndef WAVE_H
#define WAVE_H

#include "defs.h"
#include "mflow.h"

typedef struct {
  switch_t wave_forcing;
  breaking_t turb_source;
  double Hrms;
  double T;
  double theta;
  double theta_deg;
  double wepsilon_specified;   // this is the specified wave dissipation 
  double wave_num;
  double group_v, phase_v;
  double U0;
  double mepsilon;         // units m^3/s^2
  double dsxydx;           // units m^2/s^2
  double Fy;               // units m/s^2 - dsxydx/h
  int    args;
} waveinfo;



void init_wave_parameters(int nz, int nr, waveinfo *W, double scale_h, double dt, mflowinfo *M, double z0s);

void calculate_wave_stats(int time_index, double dt);

double fraction_wave_dissipation(int time_index);

void deallocate_wave_variables();

#endif /* WAVE_H */
